=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2013-10-11 05:41:32 +0000
+++ CMakeLists.txt	2014-02-18 23:01:31 +0000
@@ -1,7 +1,15 @@
 project(thumbnailer CXX)
-cmake_minimum_required(VERSION 2.8.9)
+cmake_minimum_required(VERSION 2.8.7)
+set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -Wall -pedantic -Wextra")
+
+include(CheckCXXCompilerFlag)
+check_cxx_compiler_flag(-std=c++11 HAS_C11)
+# Precise's GCC is too old to have -std=c++11
+if(HAS_C11)
 set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=c++11")
-set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -Wall -pedantic -Wextra")
+else()
+set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=c++0x -Dfinal=")
+endif()
 
 set(SO_VERSION_MAJOR "1")
 set(SO_VERSION_MINOR "0")

=== added file 'include/internal/gobj_memory.h'
--- include/internal/gobj_memory.h	1970-01-01 00:00:00 +0000
+++ include/internal/gobj_memory.h	2014-02-18 23:01:31 +0000
@@ -0,0 +1,112 @@
+/*
+ * Copyright (C) 2013 Canonical Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authored by: Jussi Pakkanen <jussi.pakkanen@canonical.com>
+ */
+
+#ifndef GOBJ_MEMORY_H_
+#define GOBJ_MEMORY_H_
+
+#include<glib-object.h>
+#include<stdexcept>
+
+/**
+ * This class is meant for automatically managing the lifetime of C objects derived
+ * from gobject. Its API perfectly mirrors the API of unique_ptr except that you
+ * can't define your own deleter function as it is always g_object_unref.
+ *
+ * API/ABI stability is not guaranteed. If you need to pass the object across an ABI
+ * boundary, pass the plain gobject.
+ *
+ * This is how you would use unique_gobj 99% of the time:
+ *
+ * unique_gobj<GSomeType> o(g_some_type_new(...));
+ *
+ * More specifically, the object will decrement the gobject reference count
+ * of the object it points to when it goes out of scope. It will never increment it.
+ * Thus you should only assign to it when already holding a reference. unique_gobj
+ * will then take ownership of that particular reference.
+ *
+ * Floating gobjects can not be put in this container as they are meant to be put
+ * into native gobject aware containers immediately upon construction. Trying to insert
+ * a floating gobject into a unique_gobj will throw an invalid_argument exception. To
+ * prevent accidental memory leaks, the floating gobject is unreffed in this case.
+ */
+template<typename T>
+class unique_gobj final {
+private:
+  T* u;
+
+  void validate_float(T *t) {
+      if(t != nullptr && g_object_is_floating(G_OBJECT(t))) {
+          throw std::invalid_argument("Tried to add a floating gobject into a unique_gobj.");
+      }
+  }
+
+public:
+  typedef T element_type;
+  typedef T* pointer;
+  typedef decltype(g_object_unref) deleter_type;
+
+  constexpr unique_gobj() noexcept : u(nullptr) {}
+  explicit unique_gobj(T *t) : u(t) {
+      // What should we do if validate throws? Unreffing unknown objs
+      // is dodgy but not unreffing runs the risk of
+      // memory leaks. Currently unrefs as u is destroyed
+      // when this exception is thrown.
+      validate_float(t);
+  }
+  constexpr unique_gobj(std::nullptr_t) noexcept : u(nullptr) {};
+  unique_gobj(unique_gobj &&o) noexcept { u = o.u; o.u = nullptr; }
+  unique_gobj(const unique_gobj &o) = delete;
+  unique_gobj& operator=(unique_gobj &o) = delete;
+  ~unique_gobj() { reset(); }
+
+  deleter_type& get_deleter() noexcept { return g_object_unref; }
+  const deleter_type& get_deleter() const noexcept { return g_object_unref; }
+
+  void swap(unique_gobj<T> &o) noexcept { T*tmp = u; u = o.u; o.u = tmp; }
+  void reset(pointer p = pointer()) {
+      if(u!=nullptr) {
+          g_object_unref(G_OBJECT(u));
+          u = nullptr;
+      }
+      // Same throw dilemma as in pointer constructor.
+      u = p;
+      validate_float(p);
+  }
+
+  T* release() noexcept { T* r = u; u=nullptr; return r; }
+  T* get() const noexcept { return u; }
+
+  T& operator*() const { return *u; }
+  T* operator->() const noexcept { return u; }
+  explicit operator bool() const noexcept { return u != nullptr; }
+
+  unique_gobj& operator=(unique_gobj &&o) noexcept { reset(); u = o.u; o.u = nullptr; return *this; }
+  unique_gobj& operator=(std::nullptr_t) noexcept { reset(); return *this; }
+  bool operator==(const unique_gobj<T> &o) const noexcept { return u == o.u; }
+  bool operator!=(const unique_gobj<T> &o) const noexcept { return u != o.u; }
+  bool operator<(const unique_gobj<T> &o) const noexcept { return u < o.u; }
+  bool operator<=(const unique_gobj<T> &o) const noexcept { return u <= o.u; }
+  bool operator>(const unique_gobj<T> &o) const noexcept { return u > o.u; }
+  bool operator>=(const unique_gobj<T> &o) const noexcept { return u >= o.u; }
+};
+
+template<typename T>
+void ::std::swap(unique_gobj<T> &f, unique_gobj<T> &s) noexcept { f.swap(s); }
+
+
+#endif

=== removed file 'include/internal/thumbnailcache.h.~1~'
--- include/internal/thumbnailcache.h.~1~	2013-10-03 03:26:49 +0000
+++ include/internal/thumbnailcache.h.~1~	1970-01-01 00:00:00 +0000
@@ -1,43 +0,0 @@
-/*
- * Copyright (C) 2013 Canonical Ltd.
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Jussi Pakkanen <jussi.pakkanen@canonical.com>
- */
-
-#ifndef THUMBNAILCACHE_H_
-#define THUMBNAILCACHE_H_
-
-#include<thumbnailer.h>
-#include<string>
-
-class ThumbnailCachePrivate;
-
-class ThumbnailCache final {
-public:
-    ThumbnailCache(const std::string &appname);
-    ~ThumbnailCache();
-
-    ThumbnailCache(const ThumbnailCache &t) = delete;
-    ThumbnailCache & operator=(const ThumbnailCache &t) = delete;
-
-    std::string get_if_exists(const std::string &abs_path, ThumbnailSize desired_size) const;
-    std::string get_cache_file_name(const std::string &as_path, ThumbnailSize desired_size) const;
-
-private:
-    ThumbnailCachePrivate *p;
-};
-
-
-#endif

=== modified file 'src/imagescaler.cpp'
--- src/imagescaler.cpp	2013-10-07 10:52:14 +0000
+++ src/imagescaler.cpp	2014-02-18 23:01:31 +0000
@@ -17,6 +17,7 @@
  */
 
 #include<internal/imagescaler.h>
+#include<internal/gobj_memory.h>
 #include<gdk-pixbuf/gdk-pixbuf.h>
 #include<memory>
 #include<stdexcept>
@@ -27,9 +28,6 @@
 
 using namespace std;
 
-static auto pbunref = [](GdkPixbuf *t) { g_object_unref(G_OBJECT(t)); };
-typedef unique_ptr<GdkPixbuf, void(*)(GdkPixbuf *t)> pb_ptr;
-
 static void determine_new_size(const int w, const int h, int &neww, int &newh,
         const ThumbnailSize wanted) {
     if(wanted == TN_SIZE_ORIGINAL) {
@@ -70,7 +68,7 @@
     GError *err = nullptr;
     string ofilename_tmp = ofilename;
     ofilename_tmp += ".tmp." + to_string(rnd());
-    pb_ptr src(gdk_pixbuf_new_from_file(ifilename.c_str(), &err), pbunref);
+    unique_gobj<GdkPixbuf> src(gdk_pixbuf_new_from_file(ifilename.c_str(), &err));
     if(err) {
         string msg = err->message;
         g_error_free(err);
@@ -84,8 +82,7 @@
 
     int neww, newh;
     determine_new_size(w, h, neww, newh, wanted);
-    pb_ptr dst(gdk_pixbuf_scale_simple(src.get(), neww, newh, GDK_INTERP_BILINEAR),
-            pbunref);
+    unique_gobj<GdkPixbuf> dst(gdk_pixbuf_scale_simple(src.get(), neww, newh, GDK_INTERP_BILINEAR));
     gboolean save_ok;
     if(original_location.empty()) {
         save_ok = gdk_pixbuf_save(dst.get(), ofilename_tmp.c_str(), "png", &err, NULL);

=== modified file 'src/thumbnailcache.cpp'
--- src/thumbnailcache.cpp	2013-10-08 12:49:51 +0000
+++ src/thumbnailcache.cpp	2014-02-18 23:01:31 +0000
@@ -16,11 +16,16 @@
  * Authored by: Jussi Pakkanen <jussi.pakkanen@canonical.com>
  */
 
+#ifndef _POSIX_SOURCE
+#define _POSIX_SOURCE
+#endif
+
 #include<thumbnailer.h>
 #include<internal/thumbnailcache.h>
 #include<stdexcept>
 #include<glib.h>
 #include<sys/stat.h>
+#include<sys/types.h>
 #include<cassert>
 #include<cstdio>
 #include<cstring>
@@ -29,6 +34,7 @@
 #include<unistd.h>
 #include<vector>
 #include<algorithm>
+#include<dirent.h>
 
 using namespace std;
 
@@ -212,7 +218,7 @@
 string ThumbnailCachePrivate::md5(const string &str) const {
     const unsigned char *buf = (const unsigned char *)str.c_str();
     char *normalized = g_utf8_normalize((const gchar*)buf, str.size(), G_NORMALIZE_ALL);
-    string final;
+    string final_result;
     gchar *result;
 
     if(normalized) {
@@ -221,10 +227,10 @@
     gssize bytes = str.length();
 
     result = g_compute_checksum_for_data(G_CHECKSUM_MD5, buf, bytes);
-    final = result;
+    final_result = result;
     g_free((gpointer)normalized);
     g_free(result);
-    return final;
+    return final_result;
 }
 
 string ThumbnailCachePrivate::get_cache_file_name(const std::string & abs_original, ThumbnailSize desired) const {
@@ -262,6 +268,16 @@
         existed = true;
         fclose(f);
     }
+    // Has it been changed since the thumbnail was taken?
+    if(existed) {
+        struct stat original, thumbnail;
+        stat(abs_path.c_str(), &original);
+        stat(fname.c_str(), &thumbnail);
+        if(original.st_mtime > thumbnail.st_mtime) {
+            p->delete_from_cache(abs_path);
+            return "";
+        }
+    }
     return existed ? fname : string("");
 }
 

=== modified file 'src/thumbnailer.cpp'
--- src/thumbnailer.cpp	2013-10-08 12:49:51 +0000
+++ src/thumbnailer.cpp	2014-02-18 23:01:31 +0000
@@ -102,6 +102,7 @@
 }
 
 Thumbnailer::Thumbnailer() {
+    g_type_init(); // Unfortunately still needed in precise.
     p = new ThumbnailerPrivate();
 }
 

=== modified file 'src/vs-thumb.cpp'
--- src/vs-thumb.cpp	2013-10-03 03:26:49 +0000
+++ src/vs-thumb.cpp	2014-02-18 23:01:31 +0000
@@ -16,6 +16,7 @@
  * Authored by: Jussi Pakkanen <jussi.pakkanen@canonical.com>
  */
 
+#include<internal/gobj_memory.h>
 #include<cstdio>
 #include<memory>
 #include<stdexcept>
@@ -51,7 +52,7 @@
             break;
     }
     // Need to preroll in order to get duration.
-    ret = gst_element_get_state(pipeline.get(), nullptr, nullptr, GST_SECOND);
+    ret = gst_element_get_state(pipeline.get(), nullptr, nullptr, 3*GST_SECOND);
     if(ret == GST_STATE_CHANGE_FAILURE) {
       throw runtime_error("Failed to preroll.");
     }
@@ -78,12 +79,11 @@
         }
         GstMapInfo mi;
         gst_buffer_map(buf, &mi, GST_MAP_READ);
-        unique_ptr<GdkPixbuf, void(*)(GdkPixbuf *t)> img(
+        unique_gobj<GdkPixbuf> img(
                 gdk_pixbuf_new_from_data(mi.data, GDK_COLORSPACE_RGB,
                         false, 8, width, height,
                         (((width * 3)+3)&~3),
-                        nullptr, nullptr),
-                        [](GdkPixbuf *t) {g_object_unref(G_OBJECT(t));});
+                        nullptr, nullptr));
         gst_buffer_unmap(buf, &mi);
         GError *err = nullptr;
         if(!gdk_pixbuf_save(img.get(), ofname.c_str(), "jpeg", &err, NULL)) {

=== modified file 'tests/CMakeLists.txt'
--- tests/CMakeLists.txt	2013-10-03 03:26:49 +0000
+++ tests/CMakeLists.txt	2014-02-18 23:01:31 +0000
@@ -8,3 +8,12 @@
 add_test(basic basic)
 set_property(TEST basic PROPERTY ENVIRONMENT "XDG_CACHE_HOME=${CMAKE_CURRENT_BINARY_DIR}")
 set_property(TEST basic APPEND PROPERTY ENVIRONMENT "TN_UTILDIR=${CMAKE_BINARY_DIR}/src")
+
+# This is only needed here because GStreamer brings in gobject.
+pkg_check_modules(GOBJ_DEPS REQUIRED gobject-2.0)
+
+include_directories(${GOBJ_DEPS_INCLUDE_DIRS})
+
+add_executable(unique_gobj unique_gobj.cpp)
+target_link_libraries(unique_gobj ${GOBJ_DEPS_LDFLAGS} ${IMG_DEPS_LDFLAGS})
+add_test(unique_gobj unique_gobj)

=== added file 'tests/unique_gobj.cpp'
--- tests/unique_gobj.cpp	1970-01-01 00:00:00 +0000
+++ tests/unique_gobj.cpp	2014-02-18 23:01:31 +0000
@@ -0,0 +1,188 @@
+/*
+ * Copyright (C) 2013 Canonical Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authored by: Jussi Pakkanen <jussi.pakkanen@canonical.com>
+ */
+
+#include<internal/gobj_memory.h>
+#include<gdk-pixbuf/gdk-pixbuf.h>
+#include<cassert>
+
+using namespace std;
+
+void trivial_test() {
+    unique_gobj<GdkPixbuf> basic(gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480));
+    assert(basic);
+    assert(gdk_pixbuf_get_width(basic.get()) == 640);
+    assert(gdk_pixbuf_get_height(basic.get()) == 480);
+}
+
+void compare_test() {
+    GdkPixbuf *pb1 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    GdkPixbuf *pb2 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    if(pb1 > pb2) {
+        std::swap(pb1, pb2);
+    }
+    assert(pb1 < pb2);
+    unique_gobj<GdkPixbuf> u1(pb1);
+    unique_gobj<GdkPixbuf> u2(pb2);
+
+    assert(!(u1 == nullptr));
+    assert(u1 != nullptr);
+    assert(u1 != u2);
+    assert(!(u1 == u2));
+    assert(u1 < u2);
+    assert(!(u2 < u1));
+    assert(!(u1 == u2));
+    assert(!(u2 == u1));
+    assert(u1 <= u2);
+    assert(!(u2 <= u1));
+}
+
+// This is its own thing due to need to avoid double release.
+
+void equality_test() {
+    GdkPixbuf *pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    unique_gobj<GdkPixbuf> u1(pb);
+    g_object_ref(G_OBJECT(pb));
+    unique_gobj<GdkPixbuf> u2(pb);
+    assert(u1 == u2);
+    assert(u2 == u1);
+    assert(!(u1 != u2));
+    assert(!(u2 != u1));
+}
+
+void release_test() {
+    GdkPixbuf *pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    unique_gobj<GdkPixbuf> u(pb);
+    assert(u);
+    assert(u.get() != nullptr);
+    assert(pb == u.release());
+    assert(!u);
+    assert(u.get() == nullptr);
+    g_object_unref(pb);
+}
+
+void sub_func(GdkPixbuf *pb) {
+    assert(G_OBJECT(pb)->ref_count == 2);
+    unique_gobj<GdkPixbuf> u(pb);
+    assert(G_OBJECT(pb)->ref_count == 2);
+    // Now it dies and refcount is reduced.
+}
+
+void refcount_test() {
+    GdkPixbuf *pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    assert(G_OBJECT(pb)->ref_count == 1);
+    g_object_ref(G_OBJECT(pb));
+    sub_func(pb);
+    assert(G_OBJECT(pb)->ref_count == 1);
+    g_object_unref(G_OBJECT(pb));
+}
+
+void swap_test() {
+    GdkPixbuf *pb1 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    GdkPixbuf *pb2 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    unique_gobj<GdkPixbuf> u1(pb1);
+    unique_gobj<GdkPixbuf> u2(pb2);
+
+    u1.swap(u2);
+    assert(u1.get() == pb2);
+    assert(u2.get() == pb1);
+
+    std::swap(u1, u2);
+    assert(u1.get() == pb1);
+    assert(u2.get() == pb2);
+}
+
+void floating_test() {
+    GdkPixbuf *pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    bool got_exception = false;
+    g_object_force_floating(G_OBJECT(pb));
+    try {
+        unique_gobj<GdkPixbuf> u(pb);
+    } catch(const invalid_argument &c) {
+        got_exception = true;
+    }
+    g_object_unref(G_OBJECT(pb));
+    assert(got_exception);
+}
+
+void move_test() {
+    GdkPixbuf *pb1 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    GdkPixbuf *pb2 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    g_object_ref(G_OBJECT(pb1));
+    unique_gobj<GdkPixbuf> u1(pb1);
+    unique_gobj<GdkPixbuf> u2(pb2);
+    u1 = std::move(u2);
+    assert(u1.get() == pb2);
+    assert(!u2);
+    assert(G_OBJECT(pb1)->ref_count == 1);
+    g_object_unref(G_OBJECT(pb1));
+}
+
+void null_test() {
+    GdkPixbuf *pb1 = NULL;
+    GdkPixbuf *pb3 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    unique_gobj<GdkPixbuf> u1(pb1);
+    unique_gobj<GdkPixbuf> u2(nullptr);
+    unique_gobj<GdkPixbuf> u3(pb3);
+
+    assert(!u1);
+    assert(!u2);
+    u3 = nullptr;
+    assert(!u3);
+}
+
+void reset_test() {
+    GdkPixbuf *pb1 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    GdkPixbuf *pb2 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 640, 480);
+    unique_gobj<GdkPixbuf> u(pb1);
+
+    u.reset(pb2);
+    assert(u.get() == pb2);
+    u.reset(nullptr);
+    assert(!u);
+}
+
+void sizeof_test() {
+    assert(sizeof(GdkPixbuf*) == sizeof(unique_gobj<GdkPixbuf>));
+}
+
+void deleter_test() {
+    unique_gobj<GdkPixbuf> u1;
+    assert(u1.get_deleter() == g_object_unref);
+}
+
+int main() {
+    g_type_init(); // Still needed in precise.
+#ifdef NDEBUG
+    fprintf(stderr, "NDEBUG defined, tests will not work.\n");
+    return 1;
+#else
+    trivial_test();
+    compare_test();
+    equality_test();
+    release_test();
+    refcount_test();
+    move_test();
+    swap_test();
+    floating_test();
+    null_test();
+    reset_test();
+    sizeof_test();
+    deleter_test();
+    return 0;
+#endif
+}

